#!/usr/bin/env python3

import pandas as pd
import gensim
from gensim.utils import simple_preprocess
from gensim.parsing.preprocessing import STOPWORDS
from nltk.stem import WordNetLemmatizer, SnowballStemmer
from nltk.stem.porter import *
import numpy as np
import nltk
import random

np.random.seed(2018)
nltk.download('wordnet')



# Load documents
# You can use only a subset, for example first 10000 documents.

data = pd.read_csv('abcnews-date-text.csv', error_bad_lines=False);
data_text = data[['headline_text']]
data_text['index'] = data_text.index
documents = data_text
print(len(documents), " documents loaded.")



# Preprocess documents - lemmatization and stemming

def lemmatize_stemming(text):
    stemmer = SnowballStemmer("english")
    return stemmer.stem(WordNetLemmatizer().lemmatize(text, pos='v'))

def preprocess(text):
    result = []
    for token in gensim.utils.simple_preprocess(text):
        if token not in gensim.parsing.preprocessing.STOPWORDS and len(token) > 3:
            result.append(lemmatize_stemming(token))
    return result

processed_docs = documents['headline_text'].map(preprocess)



# Construct dictionary

dictionary = gensim.corpora.Dictionary(processed_docs)
dictionary.filter_extremes(no_below=15, no_above=0.5, keep_n=100000)



# Filter words in documents

docs = list()
for doc in processed_docs:
    docs.append(list(filter(lambda x: x != -1, dictionary.doc2idx(doc))))



# Set the hyperparameters

iterations = 5 
topics = 10 
alpha = 0.01 
gamma = 0.0001

doc_cnt = len(docs)
wrd_cnt = len(dictionary)



# (1) Randomly initialize topics for all words in all documents.



# (2) Iterate over data and perform Gibbs sampling until convergence.



# (3) For each topic, print 10 most frequent words and their frequencies.



# (4) For the first 10 documents, show the distribution across their topics.


